import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.Timer;

/**
 *  A Java Swing component that simulates a slow keyboard processor.
 *  Each key takes one second to process, before another key will 
 *  be accepted.
 *
 *  @author Nicholas R. Howe, modified by Sara Sheehan
 *  @version February 29, 2016
 */
public class JKeyProcess extends JComponent implements ActionListener {
	
	/** For serializing the class */
	private static final long serialVersionUID = 1L;
	
    /** Currently active character */
    private Character active;

    /** Destination component for characters typed */
    private JLabel dest;

    /** Amount of processing currently finished */
    private double remaining;

    /** Time to process each character */
    private static final int TIMER_INTERVAL = 50;

    /** Keeps track of processing interval */
    private Timer timer;

    /** Keeps track of listeners */
    private List<ActionListener> listeners;

    /** 
     *  Constructor registers the label where pressed keys will go
     *  and sets up the timer
     * 
     *  @param label Place to put characters when finished processing
     */
    public JKeyProcess(JLabel dest) {
        this.dest = dest;
        this.timer = new Timer(TIMER_INTERVAL,this);
        this.remaining = 0;
        this.listeners = new ArrayList<ActionListener>();
    }

    /** 
     *  Returns true if currently busy processing
     *  @return T/F:  is active
     */
    public boolean isBusy() {
        return (this.active != null);
    }

    /** 
     *  Begins processing a key press, if one is not already active 
     *  This character will be added to the destination string one
     *  second later.  Subsequent presses during that time will be ignored.
     *
     *  @param c  The character that was pressed
     */
    public void process(char c) {
    		if (this.active == null) {
            this.active = c;
            repaint();
            this.timer.start();
            this.remaining = 1;
        } else {
            Toolkit.getDefaultToolkit().beep();
        }
    }

    /** 
     *  Timer event handler clears the active item and adds it to 
     *  the destination string.
     *
     *  @param e  The timer event
     */
    public void actionPerformed(ActionEvent e) {
        if (this.active != null) {
            this.remaining -= 0.05;
            if (this.remaining <= 0) {
                // we're finished
                this.timer.stop();
                this.dest.setText(this.dest.getText()+this.active);
                this.dest.revalidate();
                this.active = null;
                for (ActionListener l: this.listeners) {
                    l.actionPerformed(new ActionEvent(this,0,""));
                }
            }
            repaint();
        }
    }

    /**
     *  Draws the queue in the graphics window
     *
     *  @param g The graphics object to draw into
     */
    public void paintComponent(Graphics g) {
    		g.setFont(new Font("Monospace",Font.BOLD,36));
    		g.drawRect(2,2,46,46);
    		g.drawRect(3,3,44,44);
    		
        if (this.active != null) {
        		g.setColor(Color.cyan);
        		g.fillRect(5,46-(int)(41*this.remaining),41,(int)(41*this.remaining));
        		g.setColor(Color.black);
        		char arr[] = {this.active};
        		g.drawChars(arr,0,1,12,37);
        }
    }

    /**
     *  The component will look bad if it is sized smaller than this
     *  @returns The minimum dimension
     */
    public Dimension getMinimumSize() {
    		return new Dimension(50,50);
    }

    /**
     *  The component will look best at this size
     *  @returns The preferred dimension
     */
    public Dimension getPreferredSize() {
    		return new Dimension(50,50);
    }

    /**
     *  Adds an action listener to the notification list
     *  @param l  The listener to add
     */
    public void addActionListener(ActionListener listener) {
    		this.listeners.add(listener);
    }
}
